import argparse
import json
from typing import Dict, List

import paho.mqtt.client as mqtt

DISCOVERY_PREFIX = 'homeassistant'


def discovery_topic(entity_type: str, entity_id: str) -> str:
    return '%s/%s/%s/config' % (DISCOVERY_PREFIX, entity_type, entity_id)


def push_discovery(client: mqtt.Client, entity_type: str, entity_name: str, data: Dict):
    print('adding ' + entity_name)
    client.publish(
        discovery_topic(entity_type, entity_name),
        json.dumps({'name': entity_name, **data}),
        retain=True,
    )


def device(root_id: str, id: str, name: str, manufacturer: str = 'NooLite') -> Dict:
    return {
        'device': {
            'identifiers': [id],
            'manufacturer': manufacturer,
            'name': name,
            'via_device': root_id,
        }
    }


def lwt(mqtt_prefix: str) -> Dict:
    return {
        'availability_topic': '%s/LWT' % mqtt_prefix,
        'payload_available': 'Online',
        'payload_not_available': 'Offline',
    }


def push_noolite_root_device(client: mqtt.Client, mqtt_prefix: str, root_id: str):
    push_discovery(
        client, 'binary_sensor', root_id,
        {
            'name': 'NooLite MQTT Bridge Status',
            'state_topic': '%s/LWT' % mqtt_prefix,
            'payload_on': 'Online',
            'payload_off': 'Offline',
            'unique_id': root_id,
            'device_class': 'connectivity',
            'device': {
                'identifiers': [root_id],
                'manufacturer': 'NooLite',
                'name': 'NooLite MTRF64',
            },
        }
    )


def push_pm112(client: mqtt.Client, mqtt_prefix: str, root_id: str, channel: int):
    device_id = '%s_rx_%d' % (root_id, channel)
    push_discovery(
        client, 'binary_sensor', 'noolite_motion_%d' % channel,
        {
            'state_topic': '%s/switch/%d' % (mqtt_prefix, channel),
            'payload_on': 'ON',
            'device_class': 'motion',
            'off_delay': '180',
            'unique_id': '%s_motion' % device_id,
            **device(root_id, device_id, 'PM112 Motion Sensor'),
            **lwt(mqtt_prefix),
        }
    )


def push_pt111(client: mqtt.Client, mqtt_prefix: str, root_id: str, channel: int):
    device_id = '%s_rx_%d' % (root_id, channel)
    common = {
        **device(root_id, device_id, 'PT111 Temperature and Humidity Sensor'),
        **lwt(mqtt_prefix),
    }
    push_discovery(
        client, 'sensor', 'noolite_temperature_%d' % channel,
        {
            'state_topic': '%s/temperature/%d' % (mqtt_prefix, channel),
            'expire_after': 3700,
            'force_update': True,
            'device_class': 'temperature',
            'unit_of_measurement': '°C',
            'unique_id': '%s_temperature' % device_id,
            **common,
        }
    )
    push_discovery(
        client, 'sensor', 'noolite_humidity_%d' % channel,
        {
            'state_topic': '%s/humidity/%d' % (mqtt_prefix, channel),
            'expire_after': 3700,
            'force_update': True,
            'device_class': 'humidity',
            'unit_of_measurement': '%',
            'unique_id': '%s_humidity' % device_id,
            **common,
        }
    )


def push_pt112(client: mqtt.Client, mqtt_prefix: str, root_id: str, channel: int):
    device_id = '%s_rx_%d' % (root_id, channel)
    push_discovery(
        client, 'sensor', 'noolite_temperature_%d' % channel,
        {
            'state_topic': '%s/temperature/%d' % (mqtt_prefix, channel),
            'expire_after': 3700,
            'force_update': True,
            'device_class': 'temperature',
            'unit_of_measurement': '°C',
            'unique_id': '%s_temperature' % device_id,
            **device(root_id, device_id, 'PT112 Temperature Sensor'),
            **lwt(mqtt_prefix),
        }
    )


def push_pl111(client: mqtt.Client, mqtt_prefix: str, root_id: str, channel: int):
    device_id = '%s_rx_%d' % (root_id, channel)
    push_discovery(
        client, 'binary_sensor', 'noolite_light_%d' % channel,
        {
            'state_topic': '%s/switch/%d' % (mqtt_prefix, channel),
            'payload_on': 'ON',
            'payload_off': 'OFF',
            'device_class': 'light',
            'unique_id': '%s_light' % device_id,
            **device(root_id, device_id, 'PL111 Light Sensor'),
            **lwt(mqtt_prefix),
        }
    )


def push_ds1(client: mqtt.Client, mqtt_prefix: str, root_id: str, channel: int):
    device_id = '%s_rx_%d' % (root_id, channel)
    push_discovery(
        client, 'binary_sensor', 'noolite_open_%d' % channel,
        {
            'state_topic': '%s/switch/%d' % (mqtt_prefix, channel),
            'payload_on': 'ON',
            'payload_off': 'OFF',
            'device_class': 'opening',
            'unique_id': '%s_open' % device_id,
            **device(root_id, device_id, 'DS-1 Open Sensor'),
            **lwt(mqtt_prefix),
        }
    )


def push_ws1(client: mqtt.Client, mqtt_prefix: str, root_id: str, channel: int):
    device_id = '%s_rx_%d' % (root_id, channel)
    push_discovery(
        client, 'binary_sensor', 'noolite_water_%d' % channel,
        {
            'state_topic': '%s/switch/%d' % (mqtt_prefix, channel),
            'payload_on': 'ON',
            'payload_off': 'OFF',
            'device_class': 'moisture',
            'unique_id': '%s_water' % device_id,
            **device(root_id, device_id, 'WS-1 Water Sensor'),
            **lwt(mqtt_prefix),
        }
    )


def push_pxx(client: mqtt.Client, mqtt_prefix: str, root_id: str, channel: int, *,
             channel2: int = None, channel3: int = None, channel4: int = None, mode: str = 'switch'):

    device_id = '%s_rx_%d' % (root_id, channel)
    channels = [channel, channel2, channel3, channel4]

    for ch in channels:
        if ch is None:
            continue

        if mode == 'switch':
            push_discovery(
                client, 'binary_sensor', 'noolite_switch_%d' % ch,
                {
                    'state_topic': '%s/switch/%d' % (mqtt_prefix, ch),
                    'payload_on': 'ON',
                    'payload_off': 'OFF',
                    'unique_id': '%s_rx_%d_switch' % (root_id, ch),
                    **device(root_id, device_id, 'PX-XXX Remote Switch'),
                    **lwt(mqtt_prefix),
                }
            )
        elif mode == 'button':
            push_discovery(
                client, 'binary_sensor', 'noolite_button_%d' % ch,
                {
                    'state_topic': '%s/button/%d' % (mqtt_prefix, ch),
                    'payload_on': 'TOGGLE',
                    'off_delay': 1,
                    'unique_id': '%s_rx_%d_button' % (root_id, ch),
                    **device(root_id, device_id, 'PX-XXX Remote Button'),
                    **lwt(mqtt_prefix),
                }
            )
        else:
            raise TypeError('PXX mode should be either switch or button')


def push_sr1(client: mqtt.Client, mqtt_prefix: str, root_id: str, channel: int, *, mode: str = 'switch'):
    device_id = '%s_tx_%d' % (root_id, channel)
    if mode == 'switch' or mode == 'light':
        push_discovery(
            client, mode, 'noolite_switch_%d' % channel,
            {
                'command_topic': '%s/tx/%d' % (mqtt_prefix, channel),
                'unique_id': '%s_switch' % device_id,
                **device(root_id, device_id, 'SR-1-X Switch'),
                **lwt(mqtt_prefix),
            }
        )
    else:
        raise TypeError('SR1 mode should be either switch or light')


def push_su1(client: mqtt.Client, mqtt_prefix: str, root_id: str, channel: int, *, mode: str = 'light'):
    push_sr1(client, mqtt_prefix, root_id, channel, mode=mode)


def push_srf1(client: mqtt.Client, mqtt_prefix: str, root_id: str, channel: int, *, mode: str = 'switch'):
    device_id = '%s_txf_%d' % (root_id, channel)
    if mode == 'switch' or mode == 'light':
        push_discovery(
            client, mode, 'noolite_switch_%d' % channel,
            {
                'command_topic': '%s/tx-f/%d' % (mqtt_prefix, channel),
                'state_topic': '%s/state-f/%d' % (mqtt_prefix, channel),
                'unique_id': '%s_switch' % device_id,
                **device(root_id, device_id, 'SRF-1-X Switch'),
                **lwt(mqtt_prefix),
            }
        )
    else:
        raise TypeError('SRF1 mode should be either switch or light')


def push_suf1(client: mqtt.Client, mqtt_prefix: str, root_id: str, channel: int, *, mode: str = 'light'):
    push_srf1(client, mqtt_prefix, root_id, channel, mode=mode)


def push_foxel_sensor_1(client: mqtt.Client, mqtt_prefix: str, root_id: str, channel: int):
    device_id = '%s_rx_%d' % (root_id, channel)
    common = {
        'expire_after': 600,
        'force_update': True,
        **device(root_id, device_id, 'Foxel\'s NooLite Sensor', 'Foxel'),
        **lwt(mqtt_prefix),
    }
    push_discovery(
        client, 'binary_sensor', 'noolite_motion_%d' % channel,
        {
            'state_topic': '%s/switch/%d' % (mqtt_prefix, channel),
            'payload_on': 'ON',
            'payload_off': 'OFF',
            'device_class': 'motion',
            'off_delay': '180',
            'unique_id': '%s_motion' % device_id,
            **device(root_id, device_id, 'Foxel\'s NooLite Sensor', 'Foxel'),
            **lwt(mqtt_prefix),
        }
    )
    push_discovery(
        client, 'sensor', 'noolite_temperature_%d' % channel,
        {
            'state_topic': '%s/temperature/%d' % (mqtt_prefix, channel),
            'device_class': 'temperature',
            'unit_of_measurement': '°C',
            'unique_id': '%s_temperature' % device_id,
            **common,
        }
    )
    push_discovery(
        client, 'sensor', 'noolite_humidity_%d' % channel,
        {
            'state_topic': '%s/humidity/%d' % (mqtt_prefix, channel),
            'device_class': 'humidity',
            'unit_of_measurement': '%',
            'unique_id': '%s_humidity' % device_id,
            **common,
        }
    )
    push_discovery(
        client, 'sensor', 'noolite_battery_%d' % channel,
        {
            'state_topic': '%s/battery/%d' % (mqtt_prefix, channel),
            'device_class': 'battery',
            'unit_of_measurement': 'V',
            'unique_id': '%s_battery' % device_id,
            **common,
        }
    )


device_push = {
    'pm112': push_pm112,
    'pt111': push_pt111,
    'pt112': push_pt112,
    'pl111': push_pl111,
    'ds1': push_ds1,
    'ws1': push_ws1,
    'sr1': push_sr1,
    'su1': push_su1,
    'sb1': push_su1,
    'srf1': push_srf1,
    'suf1': push_suf1,
    'pxx': push_pxx,
    'fox1': push_foxel_sensor_1,
}


def send_discovery(
        devices: List[Dict],
        mqtt_host: str,
        mqtt_port: int,
        mqtt_prefix: str,
        username: str = None,
        password: str = None
):

    def on_disconnect(_client: mqtt.Client, _user_data, rc: int):
        if rc != 0:
            print("Unexpected disconnection.")

    mqtt_client = mqtt.Client()
    mqtt_client.on_disconnect = on_disconnect

    if username is not None and username != '':
        mqtt_client.username_pw_set(username, password)

    mqtt_client.connect(mqtt_host, mqtt_port, 60)

    mqtt_client.loop_start()

    root_id = mqtt_prefix.replace('/', '_')

    push_noolite_root_device(mqtt_client, mqtt_prefix, root_id)
    for device_info in devices:
        device_info = dict(device_info)
        device_type = str(device_info.pop('type'))
        device_push[device_type](mqtt_client, mqtt_prefix, root_id, **device_info)

    mqtt_client.loop_stop()
    mqtt_client.disconnect()


def json_list(value: str) -> List:
    return list(json.loads(value))


def cli():
    parser = argparse.ArgumentParser()

    parser.add_argument('mqtt_prefix', help='MQTT prefix', type=str)
    parser.add_argument('mqtt_host', help='MQTT hostname', type=str)
    parser.add_argument('username', help='MQTT user name', type=str, nargs='?', default=None)
    parser.add_argument('password', help='MQTT user password', type=str, nargs='?', default=None)
    parser.add_argument('-p', '--mqtt_port', help='MQTT port', type=int, nargs='?', default=1883)
    parser.add_argument('-d', '--devices', help='devices to discover', type=json_list, required=True)

    args = vars(parser.parse_args())

    send_discovery(**args)


if __name__ == '__main__':
    cli()



#!/usr/bin/python3

import sys
from noolite_mqtt.noolite_serial import NooLiteSerial

noo_serial = NooLiteSerial('/dev/ttyS0')

if len(sys.argv) < 3:
    raise Exception('Not Enough params')

noo_serial.send_command(int(sys.argv[1]), int(sys.argv[2]), 0)
print(noo_serial.receive())

# ch, cmd, mode, ctr
# print noo_serial.send_command(0, 15, 0, 0) #1
# noo_serial.send_command(1, 4, 2, 0) #switch
# noo_serial.send_command(0, 0, 0, 0) #turn on
# noo_serial.send_command(0, 3, 2, 0) #
# noo_serial.send_command(1, 0, 0, 0) #turn off
# noo_serial.send_command(0, 15, 2, 0) #
# noo_serial.send_command(0, 128, 2, 0,0,1) #CMD_Read_State + fmt = 1
# noo_serial.on(0)
# noo_serial.status(0)
# noo_serial.off(0)
# noo_serial.off(0)
# noo_serial.status(0)
# noo_serial.send_command(ch=0,ctr=8, cmd=4, id0=0,id1=0,id2=48,id3=114) #switch noolite ID 0.0.48.114



#!/usr/bin/python3

import sys
from noolite_mqtt.noolite_serial import NooLiteSerial

noo_serial = NooLiteSerial('/dev/ttyS0')

if len(sys.argv) < 2:
    raise Exception('Not Enough params')

noo_serial.send_command(int(sys.argv[1]), 0, mode=1, ctr=3)
print(noo_serial.receive())



import serial

from .enums import Command, Mode, Request


class NooLiteCommand:
    def __init__(
            self,
            ch: int,
            cmd: Command,
            mode: Mode = Mode.TX,
            ctr: Request = Request.CMD,
            fmt: int = 0,
            d0: int = 0, d1: int = 0, d2: int = 0, d3: int = 0,
            id0: int = 0, id1: int = 0, id2: int = 0, id3: int = 0
    ):
        self.st = 171
        self.mode = mode
        self.ctr = ctr
        self.res = 0
        self.ch = ch
        self.cmd = cmd
        self.fmt = fmt
        self.d0 = d0
        self.d1 = d1
        self.d2 = d2
        self.d3 = d3
        self.id0 = id0
        self.id1 = id1
        self.id2 = id2
        self.id3 = id3
        self.sp = 172

    @property
    def crc(self):
        crc = sum([
            self.st,
            self.mode,
            self.ctr,
            self.res,
            self.ch,
            self.cmd,
            self.fmt,
            self.d0,
            self.d1,
            self.d2,
            self.d3,
            self.id0,
            self.id1,
            self.id2,
            self.id3,
        ])
        return crc if crc < 256 else divmod(crc, 256)[1]

    def to_bytes(self):
        return bytearray([
            self.st,
            self.mode,
            self.ctr,
            self.res,
            self.ch,
            self.cmd,
            self.fmt,
            self.d0,
            self.d1,
            self.d2,
            self.d3,
            self.id0,
            self.id1,
            self.id2,
            self.id3,
            self.crc,
            self.sp
        ])


class NooLiteSerial:
    def __init__(self, tty_name):
        self.tty = self._get_tty(tty_name)

    def on(self, ch, noolite_f: bool = False):
        self.send_command(ch, Command.ON, Mode.TX_F if noolite_f else Mode.TX, Request.CMD)
        pass

    def off(self, ch, noolite_f: bool = False):
        self.send_command(ch, Command.OFF, Mode.TX_F if noolite_f else Mode.TX, Request.CMD)
        pass

    def status(self, ch):
        self.send_command(ch, Command.READ_STATE, Mode.TX_F, Request.CMD)
        pass

    def send_command(
            self,
            ch: int,
            cmd: Command,
            mode: Mode = Mode.TX,
            ctr: Request = Request.CMD,
            fmt: int = 0,
            d0: int = 0, d1: int = 0, d2: int = 0, d3: int = 0,
            id0: int = 0, id1: int = 0, id2: int = 0, id3: int = 0
    ):
        command = NooLiteCommand(ch, cmd, mode, ctr, fmt, d0, d1, d2, d3, id0, id1, id2, id3)
        self.tty.write(command.to_bytes())
        pass

    def receive(self):
        all_responses = list()
        while self.tty.inWaiting() >= 17:
            bytes_response = list(self.tty.read(17))
            if bytes_response:
                all_responses.append(bytes_response)
                if bytes_response[3] == 0:
                    break
            else:
                break
        return all_responses

    @staticmethod
    def _get_tty(tty_name):
        serial_port = serial.Serial(tty_name, 9600, timeout=0.1)
        if not serial_port.isOpen():
            serial_port.open()
        serial_port.flushInput()
        serial_port.flushOutput()
        return serial_port



import argparse
import re
import signal
from time import sleep, monotonic as time

import paho.mqtt.client as mqtt

from noolite_mqtt.noolite_serial import NooLiteSerial
from .enums import Command, Mode, Request

COMMANDS = {
    'OFF': Command.OFF,
    'ON': Command.ON,
    'SWITCH': Command.TOGGLE,
    'TOGGLE': Command.TOGGLE,
    'BIND': Command.BIND,
    'UNBIND': Command.UNBIND,
}

COMMANDS_FMT1 = {
    'BRIGHTNESS': Command.BRIGHT_SET,
}

F_COMMANDS = {
    'OFF': Command.OFF,
    'ON': Command.ON,
    'SWITCH': Command.TOGGLE,
    'TOGGLE': Command.TOGGLE,
    'BIND': Command.BIND,
    'UNBIND': Command.UNBIND,
    'GET_STATE': Command.READ_STATE,
}

F_COMMANDS_FMT1 = {
    'BRIGHTNESS': Command.BRIGHT_SET,
}

BOOLEANS = {
    '1': True,
    'ON': True,
    '0': False,
    'OFF': False,
}


class NooLiteMQTT:
    def __init__(self, serial_device: str, mqtt_host: str,
                 mqtt_port: int, mqtt_prefix: str,
                 username: str=None, password: str=None):
        self._noo_serial = NooLiteSerial(serial_device)

        self._mqtt_prefix = mqtt_prefix

        self._mqtt_client = mqtt.Client()
        self._mqtt_client.on_connect = self._on_connect
        self._mqtt_client.on_disconnect = self._on_disconnect
        self._mqtt_client.on_message = self._on_message

        if username is not None and username != '':
            self._mqtt_client.username_pw_set(username, password)

        self._postponed = []

        self._exit = False

        self._mqtt_client.will_set('%s/LWT' % self._mqtt_prefix, 'Offline', 0, True)
        self._mqtt_client.connect(mqtt_host, mqtt_port, 60)

    def loop(self):
        signal.signal(signal.SIGINT, self._interrupt_handler)
        signal.signal(signal.SIGTERM, self._interrupt_handler)

        self._mqtt_client.publish('%s/LWT' % self._mqtt_prefix, 'Online', 0, True)

        while not self._exit:
            # first receive packets from noolite serial
            packets = self._noo_serial.receive()
            for packet in packets:
                self._on_packet(packet)

            # here we work with postponed messages
            postponed = []
            for (trigger_time, topic, payload) in self._postponed:
                if trigger_time < time():
                    self._mqtt_client.publish(topic, payload)
                else:
                    postponed.append((trigger_time, topic, payload))
            self._postponed = postponed

            # here we run MQTT loop
            self._mqtt_client.loop()

    def _interrupt_handler(self, _signal, _frame):
        print('Exiting loop...')
        self._exit = True

    # The callback for when the client receives a CONNACK response
    def _on_connect(self, client: mqtt.Client, _user_data, _flags, rc: int):
        print('Connected with result code %d' % rc)

        # Subscribing in on_connect() means that if we lose the connection and
        # reconnect then subscriptions will be renewed.
        client.subscribe([
            ('%s/tx/#' % self._mqtt_prefix, 0),
            ('%s/tx-f/#' % self._mqtt_prefix, 0),
            ('%s/bind/#' % self._mqtt_prefix, 0),
            ('%s/bind-f/#' % self._mqtt_prefix, 0)
        ])

    def _on_disconnect(self, _client: mqtt.Client, _user_data, rc: int):
        self._exit = True
        if rc != 0:
            print("Unexpected disconnection.")

    # The callback for when a PUBLISH message is received from the server.
    def _on_message(self, _client: mqtt.Client, _user_data, msg: mqtt.MQTTMessage):
        print(msg.topic + ': ' + msg.payload.decode())

        tx_match = re.match('%s/tx/(\\d+)' % self._mqtt_prefix, msg.topic)
        if tx_match:
            ch = int(tx_match.group(1))
            cmd = msg.payload.decode()
            if cmd in COMMANDS:
                self._noo_serial.send_command(ch, COMMANDS[cmd], mode=Mode.TX)
                sleep(0.3)

        tx_match = re.match('%s/tx-f/(\\d+)' % self._mqtt_prefix, msg.topic)
        if tx_match:
            ch = int(tx_match.group(1))
            cmd = msg.payload.decode()
            if cmd in F_COMMANDS:
                self._noo_serial.send_command(ch, F_COMMANDS[cmd], mode=Mode.TX_F)
                sleep(0.3)

        tx_match = re.match(
            '%s/tx/(\\d+)/([A-Z]+)' % self._mqtt_prefix, msg.topic
        )
        if tx_match:
            ch = int(tx_match.group(1))
            cmd = str(tx_match.group(2))
            if cmd in COMMANDS_FMT1:
                arg = int(msg.payload.decode())
                self._noo_serial.send_command(ch, COMMANDS_FMT1[cmd], mode=Mode.TX, fmt=1, d0=arg)
                sleep(0.3)

        tx_match = re.match(
            '%s/tx-f/(\\d+)/([A-Z]+)' % self._mqtt_prefix, msg.topic
        )
        if tx_match:
            ch = int(tx_match.group(1))
            cmd = str(tx_match.group(2))
            if cmd in F_COMMANDS_FMT1:
                arg = int(msg.payload.decode())
                self._noo_serial.send_command(ch, F_COMMANDS_FMT1[cmd], mode=Mode.TX_F, fmt=1, d0=arg)
                sleep(0.3)

        # RX BIND
        bind_match = re.match('%s/bind/(\\d+)' % self._mqtt_prefix, msg.topic)
        if bind_match:
            ch = int(bind_match.group(1))
            bind_en = msg.payload.decode()
            if bind_en in BOOLEANS:
                self._noo_serial.send_command(
                    ch,
                    Command.OFF,
                    mode=Mode.RX,
                    ctr=Request.BIND_START if BOOLEANS[bind_en] else Request.BIND_STOP
                )
                sleep(0.3)

        bind_match = re.match('%s/bind-f/(\\d+)' % self._mqtt_prefix, msg.topic)
        if bind_match:
            ch = int(bind_match.group(1))
            bind_en = msg.payload.decode()
            if bind_en in BOOLEANS:
                self._noo_serial.send_command(
                    ch,
                    Command.OFF,
                    mode=Mode.RX_F,
                    ctr=Request.BIND_START if BOOLEANS[bind_en] else Request.BIND_STOP
                )
                sleep(0.3)

    # The callback to call when packet received from noolite
    def _on_packet(self, packet: bytes):
        mode = packet[1]
        ch = packet[4]
        cmd = packet[5]
        self._mqtt_client.publish(
            '%s/echo/%d' % (self._mqtt_prefix, ch),
            '[%s]' % ','.join([str(b) for b in packet])
        )
        if mode == Mode.TX_F:  # nooLite-F state case
            if cmd == Command.SEND_STATE:  # switch state
                state = packet[9] & 0x0f
                brightness = packet[10] & 0xff
                self._mqtt_client.publish(
                    '%s/state-f/%d' % (self._mqtt_prefix, ch),
                    'ON' if state > 0 else 'OFF',
                    retain=True
                )
                self._mqtt_client.publish(
                    '%s/state-f/%d/brightness' % (self._mqtt_prefix, ch),
                    str(brightness),
                    retain=True
                )
        elif mode == Mode.RX or mode == Mode.RX_F:  # regular command
            if cmd == Command.TEMPORARY_ON or cmd == Command.ON or cmd == Command.OFF:  # switch and motion detector
                switch_topic = '%s/switch/%d' % (self._mqtt_prefix, ch)
                self._mqtt_client.publish(
                    switch_topic,
                    'ON' if cmd != Command.OFF else 'OFF'
                )
                # remove any pending postponed message to this switch
                self._postponed = [
                    (trigger_time, topic, payload)
                    for (trigger_time, topic, payload) in self._postponed
                    if topic != switch_topic
                ]
                # set postponed message for motion detector
                if cmd == Command.TEMPORARY_ON:
                    interval = packet[7] * 5
                    self._postponed.append((time() + interval, switch_topic, 'OFF'))

            elif cmd == Command.TOGGLE:  # remote button
                switch_topic = '%s/button/%d' % (self._mqtt_prefix, ch)
                self._mqtt_client.publish(
                    switch_topic,
                    'TOGGLE'
                )

            elif cmd == Command.SENSOR_TEMP_HUM:  # temperature & humidity sensor
                deci_temp = packet[7] | ((packet[8] & 0x0f) << 8)

                # fix for signed 12-bit values
                if deci_temp & 0x0800:
                    from ctypes import c_int16
                    deci_temp = c_int16(deci_temp | 0xf000).value

                temp = deci_temp / 10.0
                hum = packet[9]
                battery = packet[10] / 50.0  # very custom, original PT111 sends 255 value here always
                self._mqtt_client.publish(
                    '%s/temperature/%d' % (self._mqtt_prefix, ch),
                    '%.1f' % temp
                )
                self._mqtt_client.publish(
                    '%s/humidity/%d' % (self._mqtt_prefix, ch),
                    '%d' % hum
                )
                self._mqtt_client.publish(
                    '%s/battery/%d' % (self._mqtt_prefix, ch),
                    '%.2f' % battery
                )

            elif cmd == Command.BATTERY_LOW:  # low battery
                self._mqtt_client.publish(
                    '%s/battery/%d' % (self._mqtt_prefix, ch),
                    '0'
                )


def cli():
    parser = argparse.ArgumentParser()

    parser.add_argument('serial_device', help='Serial device name', type=str)
    parser.add_argument('mqtt_prefix', help='MQTT prefix', type=str)
    parser.add_argument('mqtt_host', help='MQTT hostname', type=str)
    parser.add_argument('username', help='MQTT user name', type=str, nargs='?', default=None)
    parser.add_argument('password', help='MQTT user password', type=str, nargs='?', default=None)
    parser.add_argument('-p', '--mqtt_port', help='MQTT port', type=int, nargs='?', default=1883)

    args = vars(parser.parse_args())

    NooLiteMQTT(**args).loop()


if __name__ == '__main__':
    cli()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="noolite-mqtt",
    version="0.1.1",
    author="Andrey F. Kupreychik",
    author_email="foxel@quickfox.ru",
    description="NooLite MQTT binding",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/foxel/noolite-mqtt-python",
    packages=['noolite_mqtt'],
    install_requires=[
        'pyserial',
        'paho-mqtt',
    ],
    scripts=[
        'noolite_mqtt/noolite_cli.py',
        'noolite_mqtt/noolite_f_cli.py',
        'noolite_mqtt/noolite_rx_bind.py',
    ],
    entry_points={
        'console_scripts': [
            'noolite-mqtt=noolite_mqtt:cli',
            'noolite-mqtt-ha-discover=noolite_mqtt.hass_discover:cli',
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from enum import IntEnum


class Mode(IntEnum):
    TX = 0
    RX = 1
    TX_F = 2
    RX_F = 3
    SERVICE_F = 4
    OTA_F = 5  # firmware upgrade for NooLite-F


class Request(IntEnum):
    CMD = 0
    BROADCAST_CMD = 1
    READ = 2
    BIND_START = 3
    BIND_STOP = 4
    CLEAR = 5
    CLEAR_ALL = 6
    UNBIND = 7
    CMD_F_ADDR = 8


class Response(IntEnum):
    SUCCESS = 0
    NO_RESPONSE = 1
    ERROR = 2
    BIND_SUCCESS = 3


class Command(IntEnum):
    OFF = 0
    BRIGHT_DOWN = 1
    ON = 2
    BRIGHT_UP = 3
    TOGGLE = 4
    BRIGHT_BACK = 5
    BRIGHT_SET = 6
    LOAD_PRESET = 7
    SAVE_PRESET = 8
    UNBIND = 9
    BRIGHT_STOP = 10  # Stop_Reg
    BRIGHT_STEP_DOWN = 11
    BRIGHT_STEP_UP = 12
    BRIGHT_START = 13  # Bright_Reg
    BIND = 15
    ROLL_COLOR = 16
    RGB_COLOR = 17
    RGB_MODE = 18
    RGB_MODE_BACK = 19
    BATTERY_LOW = 20
    SENSOR_TEMP_HUM = 21
    TEMPORARY_ON = 25
    SET_MODES = 26
    READ_STATE = 128
    WRITE_STATE = 129
    SEND_STATE = 130
    SERVICE = 131
    CLEAR_MEMORY = 132
